(***********************************************************************)
(*                                                                     *)
(*                           FaCiLe                                    *)
(*                                                                     *)
(*            Nicolas Barnier, Pascal Brisset, LOG, ENAC/CENA          *)
(*                                                                     *)
(***********************************************************************)
(* $Id: magic.ml,v 1.6 2000/12/19 16:58:26 barnier Exp $ *)

(*
  Magic Sequence

  A magic sequence is a sequence of N values (x0, x1, , xN-1) such
that 0 will appear in the sequence x0 times, 1 will appear x1
times,..., and N-1 will appear in the sequence xN-1 times. For example,
for N=3, the following sequence is a solution: (1, 2, 1, 0). That is,
0 is present once, 1 is present twice, 2 is present once, and 3 is not
present.
*)

open Facile
open Easy

let go n =
  (* n variables *)
  let x = Fd.array n 0 n in

  (* Constraint: cardinality constraint with x as variables and cardinals *)
  let card_vals = Array.mapi ~f:(fun i x -> (x, i)) x in
  Cstr.post (Gcc.cstr x card_vals ~level:Gcc.Medium);

  (* Redundant constraints *)
  let vals = Array.init n ~f:(fun i -> i) in
  Cstr.post (Arith.scalprod_fd vals x =~ i2e n);
  Cstr.post (Arith.sum_fd x =~ i2e n);

  (* Search goal: first fail with min domain size *)
  let min_size = 
    Goals.GlArray.choose_index (fun a1 a2 -> Var.Attr.size a1 < Var.Attr.size a2) in
  let goal = Goals.GlArray.iter_h min_size Goals.indomain x in

  (* Search *)
  if Goals.solve goal then begin
    Array.iter ~f:(fun v -> Printf.printf "%a " Fd.fprint v) x; print_newline ()
  end
  else
    prerr_endline "No solution";;

let _ =
  if Array.length Sys.argv < 2 then
    prerr_endline "Usage: magic <size>"
  else
  go (int_of_string Sys.argv.(1));;
