(***********************************************************************)
(*                                                                     *)
(*                           FaCiLe                                    *)
(*                                                                     *)
(*            Nicolas Barnier, Pascal Brisset, LOG, ENAC/CENA          *)
(*                                                                     *)
(***********************************************************************)
(* $Id: fcl_misc.ml,v 1.5 2001/01/18 09:45:56 brisset Exp $ *)

let remove_duplicates l =
  List.fold_right ~f:(fun x l -> if List.mem x l then l else x::l) l ~init:[]


let last_and_length l =
  let rec ll n = function
    [] -> Fcl_debug.internal_error "Fcl_misc.last_and_length: empty list"
  | [x] -> (x, n)
  | _::xs -> ll (n+1) xs
  in ll 1 l


let gen_int =
  let count = ref 0 in
  fun () -> incr count; !count
  
let connexity related list =
  let rec con_rec cs = function
    [] -> cs
  | x::xs ->
     let rec x_related_with_class = function
       [] -> false
     | y::ys -> (related x y) or (x_related_with_class ys)
     in
     let rec with_x xclass = function
       [] -> [xclass]
     | c::cs -> if x_related_with_class c
                then with_x (xclass@c) cs
		else c::(with_x xclass cs)
     in con_rec (with_x [x] cs) xs
  in con_rec [] list

let rec delete element = function
  [] -> raise Not_found
| x::xs -> if x = element then xs else x::(delete element xs)


let int_overflow xy = float max_int < xy || float min_int > xy

module Operators = struct
  let ( * ) x y =
    assert (not (int_overflow (float x *. float y)));
    x * y
      
  let (+) x y =
    assert (not (int_overflow (float x +. float y)));
    x + y
      
  let (-) x y =
    assert (not (int_overflow (float x -. float y)));
    x - y

  let (=+) x y = x := !x + y

  let min (a : int) b = if a < b then a else b
  let max (a : int) b = if a < b then b else a
end

let rec iter f n z =
  if n = 0 then z else f (iter f (n-1) z);;

let rec godel f n z =
  if n = 0
  then z
  else f (n-1) (godel f (n-1) z);;


module Array = struct
  let iter2 f t1 t2 =
    assert(Array.length t1 = Array.length t2);
    for i = 0 to Array.length t1 - 1 do
      f t1.(i) t2.(i)
    done
end


let flags = (ref [] : (string * bool ref) list ref);;

let assoc_or_add n =
  try
    List.assoc n !flags
  with
    Not_found -> 
      let f = ref false in
      flags := (n, f) :: !flags;
      f;;

let protect name f =
  let already_in = assoc_or_add name in
  if !already_in then Fcl_debug.fatal_error (Printf.sprintf "%s not reentrant" name);
  already_in := true;
  try
    let x = f () in
      already_in := false;
    x
  with
    exc ->
      already_in := false;
      raise exc;;
	
